from django.shortcuts import get_object_or_404
from crea_app.models import CustomUser
from userprofile_app.models import Profile
from social_app.models import Follow, FollowRequest
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from .models import Notification
from .serializers import NotificationSerializer
from django.contrib.auth import get_user_model
from firebase_admin import messaging

User = get_user_model()

class NotificationView(APIView):
    # POST request for creating notifications
    def post(self, request, *args, **kwargs):
        # Retrieve data from the request
        action_type = request.data.get('action_type')
        user_id = request.data.get('user_id')

        # Ensure action_type and user_id are provided
        if not action_type or not user_id:
            return Response({
                "status": {
                    "type": "error",
                    "code": 400,
                    "message": "Both 'action_type' and 'user_id' are required.",
                    "data": {}
                }
            }, status=status.HTTP_400_BAD_REQUEST)
        
        # Validate the user_id and get the User object
        try:
            user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "code": 404,
                    "message": "User not found.",
                    "data": {}
                }
            }, status=status.HTTP_404_NOT_FOUND)
        
        # Determine the message based on the action type
        action_messages = {
            "liked": "Your post has been liked.",
            "commented": "Your post has been commented on.",
            "followed": "You have a new follower.",
            "viewed_profile": "Someone viewed your profile.",
            "follow_request": "You have a new follow request.",
            "added_to_casting_crew": "You have been added to the casting crew of a project.",
            "credit_request": "You have been added to the project credits or requested to join credits.",
            "reported": "You have been reported.",
            "applicant_status": "Your application status is changed.",
            "applied_users_request": "Your application has been submitted for review."
        }

        message = action_messages.get(action_type)

        if not message:
            return Response({
                "status": {
                    "type": "error",
                    "code": 400,
                    "message": "Invalid 'action_type' provided.",
                    "data": {}
                }
            }, status=status.HTTP_400_BAD_REQUEST)

        # Create the notification
        notification = Notification.objects.create(
            user=user,
            action_type=action_type,
            message=message
        )

        # if user.fcm_token:  # Assuming `fcm_token` is stored in the User model
        #     try:
        #         # Prepare the push notification
        #         push_message = messaging.Message(
        #             notification=messaging.Notification(
        #                 title="New Notification",
        #                 body=message
        #             ),
        #             token=user.fcm_token  # User's FCM token
        #         )

        #         # Send the notification
        #         response = messaging.send(push_message)
        #         print("Push notification sent successfully:", response)
        #     except Exception as e:
        #         print("Failed to send push notification:", e)

        # Return the success response with the notification details
        return Response({
            "status": {
                "type": "success",
                "code": 200,
                "message": "Notification created successfully.",
                "data": {
                    "user_id": notification.user.id,
                    "notification": notification.message
                }
            }
        }, status=status.HTTP_200_OK)

    # GET request for fetching notifications
    def get(self, request, *args, **kwargs):
        user_id = kwargs.get('user_id')  # Get user_id from URL parameters if provided
        print(user_id)
        if user_id:
            # If user_id is provided, filter notifications for that specific user
            notifications = Notification.objects.filter(user_id=user_id)
        else:
            # If user_id is not provided, return all notifications
            notifications = Notification.objects.all()

        # Check if notifications exist
        if notifications.exists():
            serializer = NotificationSerializer(notifications, many=True,context={"request": request})
            data = serializer.data
            print(data)
            for notification in data:
                if notification["action_type"]:
                    sender_id= notification["sender"]
                    print(sender_id)
                    sender_id = get_object_or_404(CustomUser, id=sender_id)
                    sender_detail = FollowRequest.objects.filter(recipient_id=user_id, requester_id = sender_id, status="pending")
                    print(sender_detail)
                    # sender_id = FollowRequest.objects.filter(recipient_id=user_id, requester_id = ).first().requester_id
                    # notification["sender_id"] = sender_id
                    profile = Profile.objects.filter(user_id=sender_id).first()
                    if profile and profile.profile_image:
                        notification["sender_profile_image"] = profile.profile_image.url
                    else:
                        notification["sender_profile_image"] = None

                    print(sender_id)
                    notification["full_name"] = sender_id.full_name
                    notification["username"] = sender_id.username

                    follower_status = Follow.objects.filter(follower=request.user, following=sender_id).exists()
                    print(follower_status)
                    following_status = Follow.objects.filter(following=request.user, follower=sender_id).exists()
                    print(following_status)
                    notification["follower_status"]=follower_status
                    notification["following_status"]=following_status
                else:
                    notification["sender_id"] = None
                    notification["sender_profile_image"] = None
                    notification["follower_status"]=None
                    notification["following_status"]=None
            return Response({
                "status": {
                    "type": "success",
                    "code": 200,
                    "message": "Notifications retrieved successfully.",
                    "data": serializer.data
                }
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "status": {
                    "type": "error",
                    "code": 404,
                    "message": "No notifications found.",
                    "data": []
                }
            }, status=status.HTTP_404_NOT_FOUND)

from firebase_admin import messaging

def send_push_notification(fcm_token, title, body):
    if not fcm_token:
        return {"status": "error", "message": "FCM token is missing"}

    message = messaging.Message(
        notification=messaging.Notification(
            title=title,
            body=body,
        ),
        token=fcm_token,
    )

    try:
        response = messaging.send(message)
        return {"status": "success", "message": "Notification sent", "response": response}
    except Exception as e:
        return {"status": "error", "message": str(e)}